#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#ifdef WIN32
#include <windows.h>
#else // for linux
#include <time.h>
#include <sys/time.h>
#endif 
///////////////////////////////////////////////////////////////////////////////////////////////////
void cpu_to_gpu();
void gpu_to_cpu();
void bench();

///////////////////////////////////////////////////////////////////////////////////////////////////

#ifdef WIN32

static LARGE_INTEGER t;
static double        f;
static int           freq_init = 0;

void timer_reset()
{
	if (!freq_init) 
	{
		LARGE_INTEGER freq;
		QueryPerformanceFrequency(&freq);
		f = (double) freq.QuadPart;
		freq_init = 1;
	  }
	
	QueryPerformanceCounter(&t);
}

static double elapse_time()
{
	LARGE_INTEGER s;
	QueryPerformanceCounter(&s);
	return ((double)(s.QuadPart - t.QuadPart)) / f;
}

#else // for linux

#define SECOND 1000000.0
static timeval reset_time;
void timer_reset()
{
	gettimeofday(&reset_time,0);
}

static double elapse_time()
{
	double current, reset;

	static struct timeval c;
	gettimeofday(&c,0);
 
 	current = c.tv_sec * SECOND + c.tv_usec;
	reset = reset_time.tv_sec * SECOND + reset_time.tv_usec;

	return (current - reset) / SECOND;
}
#endif

#define NN 2000

void timer_stop()
{
  double nflop, xmflops2, score;
  double cpu1 = elapse_time();

  nflop = (128-2)*(64-2)*(64-2)*34;

  if(cpu1 != 0.0)
    xmflops2 = nflop/cpu1*1.0e-6*(float)NN;

  score = xmflops2/32.27;
  
  printf("GPU : %f sec.\n", cpu1);
  printf("Loop executed for %d times\n",NN);
//  printf("Gosa : %e \n",gosa);
  printf("MFLOPS measured : %f\n",xmflops2);
  printf("Score based on MMX Pentium 200MHz : %f\n",score);
}

///////////////////////////////////////////////////////////////////////////////////////////////////
int main()
{
	printf("himenobench written by Ryutaro HIMENO, October 3, 1998.\n");
	printf("ported by gpgpu.jp ver.0.01 2006/05/07\n\n");

	timer_reset();
	cpu_to_gpu();
	bench();
	gpu_to_cpu();	
	timer_stop();
}

